@@ -15,6 +15,7 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
+	"math"
 	"net/http"
 	"os"
 	"sort"
@@ -51,10 +52,8 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 		addr:         desc.UsbAddr,
 		log:          NewLogger(),
 		dev:          dev,
-		connPool:     make(chan *usbConn, len(desc.IfAddrs)),
 		connReleased: make(chan struct{}),
 		shutdown:     make(chan struct{}),
-		connstate:    newUsbConnState(len(desc.IfAddrs)),
 	}
 
 	// Obtain device info
@@ -88,6 +87,7 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	for _, quirks := range transport.quirks {
 		log.Debug(' ', "  from [%s] (%s)", quirks.Model, quirks.Origin)
 		log.Debug(' ', "    blacklist = %v", quirks.Blacklist)
+		log.Debug(' ', "    usb-max-interfaces = %v", quirks.UsbMaxInterfaces)
 		for name, value := range quirks.HttpHeaders {
 			log.Debug(' ', "    http-%s = %q", strings.ToLower(name), value)
 		}
@@ -113,6 +113,8 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	log.Nl(LogDebug)
 	log.Commit()
 
+	var maxconn uint
+
 	// Check for blacklisted device
 	if len(transport.quirks) > 0 && transport.quirks[0].Blacklist {
 		err = ErrBlackListed
@@ -126,14 +128,33 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	}
 
 	// Open connections
+	maxconn = math.MaxUint32
+	for _, quirks := range transport.quirks {
+		if quirks.UsbMaxInterfaces != 0 {
+			maxconn = quirks.UsbMaxInterfaces
+		}
+	}
+
 	for i, ifaddr := range desc.IfAddrs {
 		var conn *usbConn
 		conn, err = transport.openUsbConn(i, ifaddr)
 		if err != nil {
 			goto ERROR
 		}
-		transport.connPool <- conn
+
 		transport.connList = append(transport.connList, conn)
+
+		maxconn--
+		if maxconn == 0 {
+			break
+		}
+	}
+
+	transport.connPool = make(chan *usbConn, len(transport.connList))
+	transport.connstate = newUsbConnState(len(desc.IfAddrs))
+
+	for _, conn := range transport.connList {
+		transport.connPool <- conn
 	}
 
 	return transport, nil
@@ -598,15 +619,17 @@ func (conn *usbConn) Read(b []byte) (int, error) {
 
 	// Note, to avoid LIBUSB_TRANSFER_OVERFLOW errors
 	// from libusb, input buffer size must always
-	// be aligned by 512 bytes
+	// be aligned by 1024 bytes for USB 3.0, 512 bytes
+	// for USB 2.0, so 1024 bytes alignment is safe for
+	// both
 	//
-	// However if caller requests less that 512 bytes, we
+	// However if caller requests less that 1024 bytes, we
 	// can't align here simply by shrinking the buffer,
 	// because it will result a zero-size buffer. At
-	// this case we assume caller knows what it
+	// this case we assume caller knows what it is
 	// doing (actually bufio never behaves this way)
-	if n := len(b); n >= 512 {
-		n &= ^511
+	if n := len(b); n >= 1024 {
+		n &= ^1023
 		b = b[0:n]
 	}
 
